package com.example.demo.service;

import com.example.demo.domain.Task;
import com.example.demo.domain.TrainingPhase;
import com.example.demo.dto.TaskCreateDto;
import com.example.demo.dto.TaskDto;
import com.example.demo.dto.TaskUpdateDto;
import com.example.demo.mapper.BeanMapper;
import com.example.demo.repository.TaskRepository;
import com.example.demo.repository.TrainingPhaseRepository;
import org.apache.commons.collections4.IterableUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class TaskService {

    private static final Logger LOG = LoggerFactory.getLogger(TaskService.class);

    private final TaskRepository taskRepository;
    private final TrainingPhaseRepository trainingPhaseRepository;

    @Autowired
    public TaskService(TaskRepository taskRepository, TrainingPhaseRepository trainingPhaseRepository) {
        this.taskRepository = taskRepository;
        this.trainingPhaseRepository = trainingPhaseRepository;
    }

    public TaskDto createDefaultTask(Long trainingDefinitionId, Long phaseId) {
        TrainingPhase trainingPhase = trainingPhaseRepository.findById(phaseId)
                .orElseThrow(() -> new RuntimeException("Game phase was not found"));
        // TODO throw proper exception once kypo2-training is migrated

        // TODO add check to trainingDefinitionId (field structure will be probably changed)

        Task task = new Task();
        task.setTitle("Title of task");
        task.setTrainingPhase(trainingPhase);
        task.setOrder(taskRepository.getCurrentMaxOrder(phaseId) + 1);

        Task persistedEntity = taskRepository.save(task);

        return BeanMapper.INSTANCE.toDto(persistedEntity);
    }

    public TaskDto cloneTask(Long trainingDefinitionId, Long phaseId, Long taskId) {
        Task taskToBeCloned = taskRepository.findById(taskId)
                .orElseThrow(() -> new RuntimeException("Task was not found"));
        // TODO throw proper exception once kypo2-training is migrated

        TrainingPhase trainingPhase = trainingPhaseRepository.findById(phaseId)
                .orElseThrow(() -> new RuntimeException("Game phase was not found"));

        // TODO add check to trainingDefinitionId (field structure will be probably changed)

        Task task = new Task();
        BeanUtils.copyProperties(taskToBeCloned, task);

        task.setId(null);
        task.setTrainingPhase(trainingPhase);
        task.setOrder(taskRepository.getCurrentMaxOrder(phaseId) + 1);

        Task persistedEntity = taskRepository.save(task);

        return BeanMapper.INSTANCE.toDto(persistedEntity);
    }


    public TaskDto createTask(TaskCreateDto taskCreateDto) {
        Task task = BeanMapper.INSTANCE.toEntity(taskCreateDto);

        Task persistedEntity = taskRepository.save(task);

        return BeanMapper.INSTANCE.toDto(persistedEntity);
    }

    public List<TaskDto> findAllTasks() {

        Iterable<Task> allTasks = taskRepository.findAll();

        List<TaskDto> result = new ArrayList<>();

        if (!IterableUtils.isEmpty(allTasks)) {
            for (Task task : allTasks) {
                result.add(BeanMapper.INSTANCE.toDto(task));
            }
        }

        return result;
    }

    public TaskDto getTask(Long trainingDefinitionId, Long phaseId, Long taskId) {
        Task task = taskRepository.findById(taskId)
                .orElseThrow(() -> new RuntimeException("Task was not found"));
        // TODO throw proper exception once kypo2-training is migrated

        // TODO add check to trainingDefinitionId and phaseId (field structure will be probably changed)

        return BeanMapper.INSTANCE.toDto(task);
    }

    public TaskDto updateTask(Long trainingDefinitionId, Long phaseId, TaskUpdateDto taskUpdateDto) {
        Task taskUpdate = BeanMapper.INSTANCE.toEntity(taskUpdateDto);

        Task persistedTask = taskRepository.findById(taskUpdate.getId())
                .orElseThrow(() -> new RuntimeException("Task was not found"));
        // TODO throw proper exception once kypo2-training is migrated

        // TODO add check to trainingDefinitionId and phaseId (field structure will be probably changed)

        taskUpdate.setTrainingPhase(persistedTask.getTrainingPhase());
        taskUpdate.setOrder(persistedTask.getOrder());

        Task savedEntity = taskRepository.save(taskUpdate);

        return BeanMapper.INSTANCE.toDto(savedEntity);
    }

    public void removeTask(Long trainingDefinitionId, Long phaseId, Long taskId) {
        Task task = taskRepository.findById(taskId)
                .orElseThrow(() -> new RuntimeException("Task was not found"));
        // TODO throw proper exception once kypo2-training is migrated

        // TODO add check to trainingDefinitionId and phaseId (field structure will be probably changed)

        taskRepository.decreaseOrderAfterTaskWasDeleted(phaseId, task.getOrder());

        taskRepository.delete(task);
    }
}
